//lcd.c
#include <string.h>
#include <ioaduc7020.h>
#include "lcd.h"

unsigned long data;
void DelayS (unsigned long a) { while (--a!=0); }

void E_Pulse()
{
  GP2SET |= 0x00010000; //set E to high
  DelayS(10);
  GP2CLR |= 0x00010000; //set E to low
}

void LCDInit()
{
  //first set D4, D5, D6, D7, RS, RW, E to output ports
  GP0CON  = 0x0;
  GP1CON  = 0x0;
  GP2CON  = 0x0;
  GP4CON  = 0x0;

  GP0DAT  = 0x80000000;		//RW is output
  GP1DAT  = 0xF0000000;		//D4-D7 are output
  GP2DAT  = 0x01000000;		//E is output
  GP4DAT  = 0x04000000;		//RS is output


  //LCD initialization
  //step by step (from Gosho) - from DATASHEET

  GP4DAT &= ~0x00040000;  //set RS port to 0
  GP0DAT &= ~0x00800000;  //set R/W port to 0
  GP2CLR |= 0x00010000;   //set E port to 0

  DelayS(110000);         //delay ~110ms

  GP1DAT |= 0x00300000;   //set D4 port to 1 and D5 port to 1
  GP1DAT &= 0xff3fffff;
  E_Pulse();              //high->low to E port (pulse)
  DelayS(10000);          //delay ~10ms


  GP1DAT |= 0x00300000;   //set D4 port to 1 and D5 port to 1
  GP1DAT &= 0xff3fffff;
  E_Pulse();              //high->low to E port (pulse)
  DelayS(10000);          //delay ~10ms


  GP1DAT |= 0x00300000;   //set D4 port to 1 and D5 port to 1
  GP1DAT &= 0xff3fffff;
  E_Pulse();              //high->low to E port (pulse)
  DelayS(10000);          //delay ~10ms

  GP1DAT |= 0x00200000;   //set D4 port to 1 and D5 port to 0
  GP1DAT &= 0xff2fffff;
  E_Pulse();              //high->low to E port (pulse)

}

void LCDSendCommand(unsigned long a)
{
  GP0DAT &= ~0x00800000;                  //set RW port to 0

  DelayS(2000);                           //delay for LCD char ~2ms

  data = (0x000000f0 & a) << 16;	  //get upper nibble
  GP1DAT &= 0xff0fffff;			  //clear D4-D7
  GP1DAT |= data;                         //set D4-D7
  GP4DAT &= ~0x00040000;                  //set RS port to 0 -> display set to comand mode
  E_Pulse();                              //pulse to set d4-d7 bits


  data = (0x0000000f & a) << 20;	  //get down nibble
  GP1DAT &= 0xff0fffff;			  //clear D4-D7
  GP1DAT |= data;	                  //set D4-D7
  GP4DAT &= ~0x00040000;                  //set RS port to 0 -> display set to command mode
  E_Pulse();                              //pulse to set d4-d7 bits

}

void LCDSendChar(unsigned long a)
{
    #define   RS_high     GP4DAT |= 0x00040000
#define   RS_low      GP4DAT &= ~0x00040000
#define   R_W_high    GP0DAT |= 0x00800000
#define   R_W_low     GP0DAT &= ~0x00800000

  GP0DAT &= ~0x00800000;                  //set RW port to 0

  DelayS(2000);                           //delay for LCD char ~2ms

  data = (0x000000f0 & a) << 16;	  //get upper nibble
  GP1DAT &= 0xff0fffff;			  //clear D4-D7
  GP1DAT |= data;                         //set D4-D7
  GP4DAT |= 0x00040000;                   //set RS port to 1 -> display set to data mode
  E_Pulse();                              //pulse to set d4-d7 bits

  data = (0x0000000f & a) << 20;	  //get down nibble
  GP1DAT &= 0xff0fffff;			  //clear D4-D7
  GP1DAT |= data;	                  //set D4-D7
  GP4DAT |= 0x00040000;                   //set RS port to 1 -> display set to data mode
  E_Pulse();                              //pulse to set d4-d7 bits

}

void LCDSendTxt(char* a)
{
  for(int i=0; i<strlen(a); i++)
  {
    LCDSendChar(a[i]);
  }
}

void SmartUp(void)
{
  for(int i=0; i<40; i++) LCDSendCommand(CUR_RIGHT);
}

void SmartDown(void)
{
  for(int i=0; i<40; i++) LCDSendCommand(CUR_LEFT);
}

void Light(short a)
{
  if(a == 1)
  {

  }
  if(a == 0)
  {

  }
}